import logging
import psycopg2 as pg
import time

logger = logging.getLogger(__name__)


class ConnectBaaSDB:

    def __init__(self, parameters):
        ''' Initialize the class '''
        self.parameters = parameters
        self.connection = None
        self.runtime = None

    def connect_to_db(self):
        ''' Connect to the Postgres database
            with a maximum of 3 attempts '''
        attempts = 1

        while attempts < 4:
            logger.info('Connecting to Postgres database: attempt {0} of 3'.format(attempts))
            try:
                self.connection = pg.connect(**self.parameters)
                logger.info('Connected to Postgres database')
                break
            except pg.Error as error:
                logger.error(error)
                time.sleep(3)
            finally:
                attempts += 1

    def close_connection_to_db(self):
        ''' Close the connection to the Postgres database '''
        attempts = 0

        while self.connection.closed == 0 and attempts < 3:
            logger.info('Closing connection to Postgres database')
            try:
                self.connection.close()
                logger.info('Connection to Postgres database is closed')
                break
            except pg.Error as error:
                logger.error(error)
                time.sleep(3)
            finally:
                attempts += 1

    def get_runtime_parameters(self, component):
        ''' Use this function to retrieve the parameters for each component of the Postgres Database '''
        logger.info('Retrieve runtime parameters for {}'.format(component))

        # Create a cursor object
        logger.debug('Creating cursor object')
        cursor = self.connection.cursor()

        try:
            logger.debug('Executing baas_own.baas_get_runtime_parameters')
            cursor.callproc('baas_own.baas_get_runtime_parameters', (component,))

            logger.debug('Fetching all results from query')
            all_rows = cursor.fetchall()
            self.runtime = dict(all_rows)

        except pg.DatabaseError as error:
            logger.error(error)

        finally:
            logger.debug('Closing cursor object')
            cursor.close()

    def update_runtime_parameters(self, component, parameter, input_variable):
        """ This function is used to update the config_runtime table in the Postgres database
        """
        logger.info('Start of updating the runtime parameters')

        # Define the query
        logger.debug('Defining the query')
        if input_variable == 'NULL':
            query = f"""UPDATE config_runtime SET "target_value" = {input_variable} WHERE "component" = '{component}'""" \
                    f""" AND "parameter" = '{parameter}';"""
        else:
            query = f"""UPDATE config_runtime SET "target_value" = '{input_variable}' WHERE "component" = '{component}'""" \
                    f""" AND "parameter" = '{parameter}';"""

        # Create a cursor
        logger.debug('Creating the cursor object')
        cursor = self.connection.cursor()

        try:
            # Execute the query using the cursor object
            logger.debug(f'Executing {query}')
            cursor.execute(query)

            # Commit the pending transaction to the database
            logger.debug(f'Committing the pending transaction {cursor.query}')
            self.connection.commit()

        except pg.Error as error:
            # Produce an error log
            logger.error(error)

            # Rollback the pending transaction to the database
            logger.debug(f'Rolling back the pending transaction {cursor.query}')
            self.connection.rollback()

        finally:
            # Close the cursor object
            logger.debug('Closing the cursor object')
            cursor.close()

        logger.info('End of updating the runtime parameters')





import yaml


def open_yaml(filename):
    with open(filename, 'r') as stream:
        try:
            config = yaml.safe_load(stream)
        except yaml.YAMLError as error:
            raise error

    return config


import logging.handlers
import logging
import json
import psycopg2 as pg
import uuid

from connect_baas_db.connect_baas_db import ConnectBaaSDB


class PostgresHandler(logging.Handler, ConnectBaaSDB):
    '''
    Customized logging handler that puts logs to the database.
    '''

    def __init__(self, parameters):
        logging.Handler.__init__(self)
        ConnectBaaSDB.__init__(self, parameters)
        self.connect_to_db()
        self.reference_id = str(uuid.uuid4())
        self.name = 'PostgresHandler'

    def emit(self, record):
        if self.connection:
            cursor = self.connection.cursor()

            log_msg = str(record.msg)
            log_msg = log_msg.strip()
            log_msg = log_msg.replace('\'', '\'\'')
            message = str(log_msg)

            application = str(record.name)
            module = str(record.module)
            func = str(record.funcName)
            severity = str(record.levelname)

            json_message = json.dumps({'referenceid': self.reference_id,
                                       'application': application,
                                       'module': module,
                                       'function': func,
                                       'severity': severity,
                                       'message': message})

            query = f"CALL baas_own.baas_logger('{self.reference_id}', '{application}', '{module}.{func}', '{severity}', '{message}', '{json_message}');"

            attempts = 0
            while attempts < 3:
                try:
                    cursor.execute(query)
                    self.connection.commit()
                    cursor.close()
                    break
                except pg.DatabaseError:
                    self.connection.rollback()
                finally:
                    attempts += 1
            else:
                cursor.close()
                self.close_connection_to_db()



from setuptools import setup, find_packages

setup(
    name='connect_baas_db',
    version='1.0.0',
    description='A Python package with the intend of connecting to a Postgres database',
    long_description='This package can be used to simplify connecting to Postgres database and logging to the Postgres database. Furthermore a functionality for opening yaml files has been added. This package has been created for specific purposes',
    url='',
    author='Sander van Dessel',
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 3 - Alpha',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate you support Python 3. These classifiers are *not*
        # checked by 'pip install'. See instead 'python_requires' below.
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3 :: Only',
    ],
    package_dir={'': 'src'},
    packages=find_packages(where='src'),
    python_requires='>=3.6, <4',
    install_requires=['psycopg2-binary', 'pyyaml']
)
